      program micros
      implicit none

! educational version of an s-process network.
! one hundred isotopes are included and the initial abundance of the first isotope,
! nominally fe56, is taken equal to one. these abudances are evolved
! until the user chosen ending time.
! 
! a little theory on what this program does:
! 
! let "ratng" be the reaction rate for (n,g) reactions.
! the ordinary differential equations describing the change in the abundances y are:
! dydt(1) = -ratng(1) * y(1)                         i = 1 
! dydt(i) =  ratng(i-1) * y(i-1) - ratng(i) * y(i)   i=2,n-1
! dydt(n) =  ratng(n-1) * y(n-1)                     i = n
! 
! for the first-order accurate euler method,
! each abundance is updated over a timestep h as
! ynew(i) = y(i) + dy(i)
! where the change dy(i) is obtained from solving the linear system
!  (I/h - J) = f(i)
! 
! with only (n,g) reactions, the jacobian matrix J has the special form
! 
! | -ratng(1)                                                 |
! |  ratng(1)  -ratng(2)                                      |
! |             ratng(2)    -ratng(3)                         |
! |                                     ...                   |
! |                                           ratng(n-1)  0.0 |
! 
! this system of linear equations can be easily solved by hand:
!  dy(1) = -y(1) * ratng(1) / (dt + ratng(1)
!  dy(i) = (-y(i)*ratng(i) + (y(i-1) + dy(i-1))*ratng(i-1))/(1/h + ratng(i))
!  dy(n) = y(n-1)*ratng(n-1)*h
! 
! hence the very succint updating scheme implemented below.


! declare
      character*20       :: outfile,rootfile
      integer            :: i,j
      integer, parameter :: jmax=10000, nmax=100
      real*8             :: y(nmax),dy(nmax),ratng(nmax),tend,dt,dtinv,t,xx,yout(nmax)



! time step controllers
! chimin is smallest abundance that can affect the time step
! delchi is maximum fractional change allowed
! fdtn is the maximum enlargement factor for new time step
! in english this says don't let any abundance change by more than 5%
! if its abundnace is larger that 0.001. 

      real*8, parameter :: chimin = 1.0d-3, &
                           delchi = 0.05d0, &
                           fdtn   = 2.0d0


! initialize, this effectively says all the cross sections are equal.
      y     = 0.0d0
      ratng = 1.0d0


! as an exercise in bottlenecks, try setting a few not equal to one as shown below
      y(1) = 1.0d0
      rootfile ='all_equal_'

!      ratng(nmax/2) = 0.1d0
!      rootfile ='mid_0p1_'

!      ratng(nmax/2) = 10.0d0
!      rootfile ='mid_10p0_'


! get the ending time. note the ending time may be interpreted as the
! exposure tau, the integral of the neutron density, velocity, and time.

! starting time, ending time and initial timestep
      t    = 0.0d0
      tend = 100.0d0
      dt   = 1.0d-6*tend


! start the evolution loop
      do j=1,jmax


! increment the total time
       t = t + dt


! a very succint update scheme
       dtinv = 1.0d0/dt

       dy(1) = -y(1)*ratng(1)/(dtinv + ratng(1))
       y(1)  = y(1) + dy(1)

       do i=2,nmax-1
        dy(i) = (-y(i)*ratng(i) + y(i-1)*ratng(i-1))/(dtinv + ratng(i))
        y(i)  = y(i) + dy(i)
       enddo

       dy(nmax) = y(nmax-1)*ratng(nmax-1)*dt
       y(nmax)  = y(nmax) + dy(nmax)


! write what we got for this time step
        yout = max(y,1.0e-16)
        write(outfile,'(a,i4.4,a)') trim(rootfile),j,'.dat'
        open(unit=2,file=outfile,status='unknown')

        write(2,'(a,1pe11.3)') 'time =', t
        write(6,'(a,1pe11.3)') 'time =', t
        write (2,'(a,a)') '   i', '   abundance'
        write (2,'(i5,1pe11.3)') (i,yout(i), i=1,nmax)

        close(unit=2)


! check if we are done 
       if ( t-tend .ge. 0.0d0  .or.  j .eq. jmax ) exit

! otherwise, get a new timestep
       xx = fdtn*dt
       do i = 1,nmax
        if (y(i) .gt. chimin) then
         xx = min(xx,abs(y(i)/dy(i))*delchi*dt)
        endif
       enddo
       dt = xx


! if the step can overshoot the stop point, cut it down
       if ( t+dt-tend .gt. 0.0d0) dt = tend - t


! end the evolution loop
      enddo

      end program micros
